#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#include "libps.h"


#define PSVisualGrey	0
#define PSVisualRGB	1
#define PSVisualRGBA	2


static int PSWriteFilePS3Any(
        FILE *fp, PSFormatStruct *fmt,
        int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
	void *client_data,		/* Client data for progress callback. */
	int (*progress_cb)(void *, long, long),	/* Progress callback func. */
	int visual
);

int PSWriteFilePS3RGBA(
        FILE *fp, PSFormatStruct *fmt,
        int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
        void *client_data,              /* Client data for progress callback. */
        int (*progress_cb)(void *, long, long)	/* Progress callback func. */
);
int PSWriteFilePS3RGB(
        FILE *fp, PSFormatStruct *fmt,
        int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
        void *client_data,              /* Client data for progress callback. */
        int (*progress_cb)(void *, long, long)	/* Progress callback func. */
);
int PSWriteFilePS3Grey(
        FILE *fp, PSFormatStruct *fmt,
        int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
        void *client_data,              /* Client data for progress callback. */
        int (*progress_cb)(void *, long, long)	/* Progress callback func. */
);


/*
 *	Mexus procedure to write a postscript file.
 *
 *	The visual can be one of the following codes:
 *
 *	PSVisualGrey		1 bpp
 *	PSVisualRGB		3 bpp
 *	PSVisualRGBA		4 bpp
 *
 *	Returns 0 on success, or non-zero on error.
 */
static int PSWriteFilePS3Any(
	FILE *fp, PSFormatStruct *fmt,
	int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
        void *client_data,		/* Client data for progress callback. */
        int (*progress_cb)(void *, long, long),	/* Progress callback func. */
	int visual
)
{
	PSFlags flags;
	int user_aborted = 0;
	int bpp = 1;			/* Bytes per pixel of given buffer. */
	int bpp_out = 1;		/* Bytes per pixel of output data. */
	int dpi_x = PSDefaultDPIX, dpi_y = PSDefaultDPIY;
	int paper_width = 320, paper_height = 240;
	int paper_widthm, paper_heightm;
	int output_x = 0, output_y = 0;


	/* Get values from format structure. */
	flags = fmt->flags;

        if(flags & PSFlagDPIX)
            dpi_x = fmt->dpi_x;
        if(flags & PSFlagDPIY)
            dpi_y = fmt->dpi_y;

	if(flags & PSFlagPaperWidth)
	    paper_width = fmt->paper_width;
	if(flags & PSFlagPaperHeight)
	    paper_height = fmt->paper_height;

        if(flags & PSFlagOutputX)
            output_x = fmt->output_x;
        if(flags & PSFlagOutputY)
            output_y = fmt->output_y;

	/* Calculate paper size with DPI applied. */
	paper_widthm = (int)(paper_width * dpi_x / 100);
        paper_heightm = (int)(paper_height * dpi_y / 100);


	/* Determine bytes per pixel from visual. */
        switch(visual)
	{
	  case PSVisualRGBA:
	    bpp = 4;
	    bpp_out = 3;
	    break;

	  case PSVisualRGB:
	    bpp = 3;
	    bpp_out = 3;
	    break;

	  case PSVisualGrey:
	    bpp = 1;
	    bpp_out = 1;
            break;

	  default:
	    return(PSErrorInternal);
	    break;
	}

	/* If the given bytes per line bpl is 0 then that implies they
	 * want us to calculate it. In which we will assume it is
	 * width * bpp.
	 */
	if(bpl <= 0)
	    bpl = width * bpp;
	if(bpl < 1)
	    return(PSBadValue);


	/* Call progress callback initially. */
	if(!user_aborted && (progress_cb != NULL))
	{
	    if(progress_cb(client_data, 0, 1))
		user_aborted = 1;
	}

	/* Begin writing postscript data to the stream fp. */

	/* Header. */

	/* Format identifier. */
        fprintf(
	    fp,
"%%!PS-Adobe-3.0\n"
	);
        /* Creator. */
        fprintf(
            fp,
"%%%%Creator: %s\n",
            "Generic PostScript Generator"
        );
	/* Title. */
        fprintf(
            fp,
"%%%%Title: %s\n",
	    "Untitled"
        );
	/* Date. */
/*
        fprintf(
            fp,
"%%%%CreationDate: ???\n",
            ???
        );
 */
	/* Document data type. */
        fprintf(
            fp,
"%%%%DocumentData: Clean7Bit\n"
        );

	/* Pages. */
        fprintf(
            fp,
"%%%%Pages: %i\n",
            1
        );

	/* Bounding box (in units of pixels). */
        fprintf(
            fp,
"%%%%BoundingBox: %i %i %i %i\n",
            0, 0,
            paper_widthm - 1, paper_heightm - 1
        );

	/* End comments. */
        fprintf(
            fp,
"%%%%EndComments\n"
	);


	/* Begin prolong. */
	fprintf(
            fp,
"%%%%BeginProlog\n\n"
	);

        fprintf(
            fp,
"%% Use own dictionary to avoid conflicts\n"
        );
        fprintf(
            fp,
"5 dict begin\n"
        );

        /* End prolong. */
        fprintf(
            fp,
"%%%%EndProlog\n\n"
        );



	/* Page set. */
	fprintf(
            fp,
"%%%%Page: %i %i\n\n",
	    1, 1
        );

	/* Translate for offset. */
        fprintf(
            fp,
"%% Translate for offset (target at lower-left corner)\n"
        );     
        fprintf(
            fp,
"%i %i translate\n",
            output_x,
            paper_heightm - output_y - height
        );

	/* Scale. */
	fprintf(
            fp,
"%% Scale of image from unit size\n"
        );
        fprintf( 
            fp,
"%.5f %.5f scale\n\n",
            (float)width, (float)height
        );

	/* Scan line buffer. */
        fprintf(
            fp,
"%% Buffer to store one scan line (width * bytes_per_pixel)\n"
        );
        fprintf(
            fp,
"/scanline %i %i mul string def\n",
	    width, bpp_out
        );

	/* Image geometry. */
        fprintf(
            fp,
"%% Image geometry\n"
        );
        fprintf(
            fp,
"%i %i %i\n",
            width, height, 8
        );

        /* Transformatino matrix. */
        fprintf(
            fp,
"%% Transformation matrix\n"
        );
        fprintf(
            fp,
"[ %i %i %i %i %i %i ]\n",
            width,
	    0,
	    0,
	    height * -1,
	    0,
	    height
        );


	switch(visual)
	{
	  case PSVisualRGBA:
          case PSVisualRGB:
	    fprintf(
		fp,
"{ currentfile scanline readhexstring pop } false 3\n"
	    );
	    fprintf(
		fp,
"colorimage\n"
	    );
	    break;

	  default:	/* PSVisualGrey. */
            fprintf(
                fp,
"{ currentfile scanline readhexstring pop }\n"
	    );
            fprintf(
                fp,
"image\n"
	    );
	    break;
	}

	/* Begin writing image data. */
	if(!user_aborted)
	{
	    int x, y;
	    int col = 0, line_max = 70;
	    long progress_max, progress_cur;
	    const u_int8_t *src_buf, *src_ptr;


	    /* Calculate progress bounds (using units of pixels since
	     * is easilly obtained from the itterations through the
	     * output image.
	     */
	    progress_cur = 0;
	    progress_max = width * height;


	    /* Get pointer to base of source image data. */
	    src_buf = (const u_int8_t *)buf;

	    /* Itterate through pixels. */
            for(y = 0; y < height; y++)
            {
		/* Call progress callback? */
		if(progress_cb != NULL)
		{
		    progress_cur = (y * width);

		    /* Call progress callback and check for abort return. */
		    if(progress_cb(client_data, progress_cur, progress_max))
		    {
			user_aborted = 1;
			break;
		    }
		}

                for(x = 0; x < width; x++)
                {
		    /* Get pointer to source pixel. */
		    src_ptr = &src_buf[
			(y * bpl) + (x * bpp)
		    ];

                    /* Write pixel depending on visual. */
                    switch(visual)
		    {
		      case PSVisualRGBA:
			/* Red. */
                        fprintf(fp, "%.2x",
                            src_ptr[0]
			);
			/* Green. */
                        fprintf(fp, "%.2x",
			    src_ptr[1]
			);
			/* Blue. */
                        fprintf(fp, "%.2x",
                            src_ptr[2]
                        );
			/* Skip alpha. */

                        col += 6;
			break;

                      case PSVisualRGB:
                        /* Red. */
                        fprintf(fp, "%.2x",
                            src_ptr[0]
                        );
                        /* Green. */
                        fprintf(fp, "%.2x",
                            src_ptr[1]
                        );
                        /* Blue. */
                        fprintf(fp, "%.2x",
                            src_ptr[2]
                        );

                        col += 6;
                        break;

                      case PSVisualGrey:
                        /* Grey. */
                        fprintf(fp, "%.2x",
                            src_ptr[0]
                        );

                        col += 2;
                        break;
		    }

		    /* Columns exceeded? */
                    if(col >= line_max)
                    {
                        col = 0;
                        fputc('\n', fp);
                    }
                }

                col = 0;
                fputc('\n', fp);
            }
        }

        fputc('\n', fp);


        /* Footer. */
	fprintf(
	    fp,
"showpage\n\
%%%%Trailer\n\
end\n\
%%%%EOF\n"
	);


	/* Notify progress callback on completion. */
        if(!user_aborted && (progress_cb != NULL))
	{
	    if(progress_cb(client_data, 1, 1))
		user_aborted = 1;
	}

	return(user_aborted ? PSAbort : PSSuccess);
}


/*      Writes PS format data to the stream fp.
 *
 *      The image data buf is given in RGBA format (4 bytes per pixel).
 *      The bytes per line bpl can be 0 which means width * 4.
 *
 *      The progress_cb specifies the progress callback, this function is
 *      called during writing and the given client_data is passed to it.
 *      A return value of 0 means keep processing while a non-zero means
 *      abort the operation.
 *
 *      Returns PSSuccess on success or PSError on error. Can also return
 *      PSBadValue if the given values in fmt are unacceptable.
 */
int PSWriteFilePS3RGBA(
        FILE *fp, PSFormatStruct *fmt,
        int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
        void *client_data,              /* Client data for progress callback. */
        int (*progress_cb)(void *, long, long)	/* Progress callback func. */
)
{
	if((fp == NULL) || (fmt == NULL) || (buf == NULL))
	    return(PSBadValue);
	if((width < 1) || (height < 1))
	    return(PSBadValue);

	return(
	    PSWriteFilePS3Any(
		fp, fmt, width, height, bpl, buf,
		client_data, progress_cb,
		PSVisualRGBA
	    )
	);
}

/*      Writes PS format data to the stream fp.
 *
 *      The image data buf is given in RGB format (3 bytes per pixel).
 *      The bytes per line bpl can be 0 which means width * 3.
 *
 *      The progress_cb specifies the progress callback, this function is
 *      called during writing and the given client_data is passed to it.
 *      A return value of 0 means keep processing while a non-zero means
 *      abort the operation.
 *
 *      Returns PSSuccess on success or PSError on error. Can also return
 *      PSBadValue if the given values in fmt are unacceptable.
 */
int PSWriteFilePS3RGB(
        FILE *fp, PSFormatStruct *fmt,
        int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
        void *client_data,              /* Client data for progress callback. */
        int (*progress_cb)(void *, long, long)	/* Progress callback func. */
)
{
        if((fp == NULL) || (fmt == NULL) || (buf == NULL))
            return(PSBadValue);
        if((width < 1) || (height < 1))
            return(PSBadValue);

        return(
            PSWriteFilePS3Any(
                fp, fmt, width, height, bpl, buf,
		client_data, progress_cb,
                PSVisualRGB
            )
        );
}

/*      Writes PS format data to the stream fp.
 *
 *      The image data buf is given in greyscale format (1 byte per pixel).
 *      The bytes per line bpl can be 0 which means just width.
 *
 *	The progress_cb specifies the progress callback, this function is
 *	called during writing and the given client_data is passed to it.
 *	A return value of 0 means keep processing while a non-zero means
 *	abort the operation.
 *
 *      Returns PSSuccess on success or PSError on error. Can also return
 *      PSBadValue if the given values in fmt are unacceptable.
 */
int PSWriteFilePS3Grey(
        FILE *fp, PSFormatStruct *fmt,
        int width, int height,          /* Size of image in pixels. */
        int bpl,                        /* Bytes per pixel (or row stride). */
        const void *buf,
        void *client_data,              /* Client data for progress callback. */
        int (*progress_cb)(void *, long, long)	/* Progress callback func. */
)
{
        if((fp == NULL) || (fmt == NULL) || (buf == NULL))
            return(PSBadValue);
        if((width < 1) || (height < 1))
            return(PSBadValue);

        return(
            PSWriteFilePS3Any(
                fp, fmt, width, height, bpl, buf,
		client_data, progress_cb,
                PSVisualGrey
            )
        );
}
