/*
                          Print Options Dialog

	For use in conjunction with the image viewer.
 */

#ifndef PRINTOPTS_H
#define PRINTOPTS_H

#include <sys/types.h>
#include <gtk/gtk.h>
#include "imgview.h"

#ifndef gbool
# define gbool	guchar
#endif


/*
 *	Print output types:
 */
#define PRINT_OUTPUT_TO_PRINTER	0
#define PRINT_OUTPUT_TO_FILE	1

/*
 *	Visual:
 */
#define PRINT_VISUAL_GREYSCALE	0
#define PRINT_VISUAL_COLOR	1

/*
 *	Orientation:
 */
#define PRINT_ORIENTATION_PORTRAIT		0
#define PRINT_ORIENTATION_LANDSCAPE		1

/*
 *	Paper size list, each entry contains four values:
 *
 *	char *name, char *dimensions, int width, int height, int units
 *
 *	Where width and height are in pixels at 100 dpi.
 */
#define PAPER_SIZE_LIST		{ \
	"Letter",	"8.5 x 11 in",	(gchar *)850,	(gchar *)1100, \
	"Tabloid",	"11 x 17 in",	(gchar *)1100,	(gchar *)1700, \
	"Ledger",	"17 x 11 in",	(gchar *)1700,	(gchar *)1100, \
        "Legal",	"8.5 x 14 in",	(gchar *)850,	(gchar *)1400, \
	"Executive",	"7.5 x 10 in",	(gchar *)750,	(gchar *)1000, \
	"A3",		"11.69 x 16.53 in",	(gchar *)1169, (gchar *)1653, \
	"A4",		"8.26 x 11.69 in",	(gchar *)826, (gchar *)1169, \
	"A5",		"5.83 x 8.26 in", 	(gchar *)583, (gchar *)826, \
	"B4",		"9.83 x 13.93 in",	(gchar *)983, (gchar *)1393, \
	"B5",		"7.17 x 10.12 in",	(gchar *)717, (gchar *)1012, \
	NULL, NULL, NULL, NULL	/* Last entry is 4 NULL's. */ \
}


/*
 *	Print values structure:
 */
typedef struct {

	/* Flags that indicate which members of this function are
	 * defined.
	 */
#define PRINT_VALUE_OUTPUT_TYPE		(1 << 0)
#define PRINT_VALUE_PRINT_COMMAND	(1 << 1)
#define PRINT_VALUE_FILE_NAME		(1 << 2)
#define PRINT_VALUE_BRIGHTNESS		(1 << 3)
#define PRINT_VALUE_CONTRAST		(1 << 4)
#define PRINT_VALUE_VISUAL		(1 << 5)
#define PRINT_VALUE_ORIENTATION		(1 << 6)
#define PRINT_VALUE_DPI_X		(1 << 7)
#define PRINT_VALUE_DPI_Y		(1 << 8)
#define PRINT_VALUE_PAPER_WIDTH		(1 << 9)
#define PRINT_VALUE_PAPER_HEIGHT	(1 << 10)
#define PRINT_VALUE_OUTPUT_X		(1 << 11)
#define PRINT_VALUE_OUTPUT_Y		(1 << 12)
#define PRINT_VALUE_OUTPUT_WIDTH	(1 << 13)
#define PRINT_VALUE_OUTPUT_HEIGHT	(1 << 14)
	guint flags;

	gint output_type;	/* One of PRINT_OUTPUT_*. */
	gchar *print_command;	/* Do not modify or deallocate, can be NULL. */
	gchar *file_name;	/* Do not modify or deallocate, can be NULL. */
	gfloat brightness;	/* -1.0 to 1.0. */
	gfloat contrast;	/* -1.0 to 1.0. */
	gint visual;		/* One of PRINT_VISUAL_*. */
	gint orientation;	/* One of PRINT_ORIENTATION_*. */
	gint dpi_x, dpi_y;
	gint paper_width, paper_height;	/* In pixels, without dpi applied. */
	/* Note output geometry matches upper left portrait orientation.
	 * Size will already be modified for correct orientation.
	 */
	gint output_x, output_y;	/* In pixels, from upper left. */
	gint output_width, output_height;


} print_values_struct;


/*
 *	Print dialog structure:
 */
typedef struct {

	gbool initialized;
	gbool map_state;
	GtkAccelGroup *accelgrp;
	GtkWidget *toplevel;

	/* Cursors. */
	GdkCursor	*busy_cur,
			*translate_cur,
			*size_cur;

	/* Pointer back to image viewer. */
	imgview_struct *iv;

	/* Output radios. */
	GtkWidget	*output_printer_radio,
			*output_file_radio;
	/* Output entries. */
	GtkWidget	*output_printer_entry,	/* Print command. */
			*output_file_entry;	/* File name. */
	GtkWidget	*output_file_browse_btn;

	/* Visual scales. */
        GtkWidget	*brightness_scale;
        GtkAdjustment	*brightness_adj;
        GtkWidget	*contrast_scale;
        GtkAdjustment	*contrast_adj;
	/* Visual radios. */
	GtkWidget	*visual_greyscale_radio,
			*visual_color_radio;

	/* Orientation radios. */
	GtkWidget	*orientation_portrait_radio,
			*orientation_landscape_radio;

	/* DPI entries, axis relative to paper (not image). */
	GtkWidget	*dpi_x_entry,
			*dpi_y_entry;

	/* Paper size clist. */
	GtkWidget	*paper_size_clist;


	/* Output geometry entries. */
	GtkWidget	*output_x_entry,
			*output_y_entry,
			*output_width_entry,
			*output_height_entry;
	/* If this is TRUE then the "changed" signal sent to the above
	 * output entry widgets will be ignored, this is used for manually
	 * updating the entries within the program.
	 */
	gbool output_entry_update_manual;

	/* Preview drawing area. */
	GtkWidget	*preview_da;

	/* Graphic contexts for the preview drawnig area. */
	GdkGC *gc_base, *gc_invert;

	/* Preview image, this image is post processed with the correct
	 * window coordinate size, color, brightness, rotations, etc and
	 * ready to be drawn on the preview_da.
	 */
	imgview_image_struct *preview_img;

	/* Original image, this image contains the original data recieved
	 * from the image viewer. It has only the correct window
	 * coordinate size and rotations, but not brightness or
	 * contrast. This is so that it can be quickly coppied to the
	 * preview_img (since sizes match) with modifications to contrast
	 * and brightness.
	 */
	imgview_image_struct *orig_preview_img;

	/* Preview menu. */
	GtkWidget *menu;

	/* Image output geometry (not allocated size of preview image), in
	 * pixels. Relative to upper left corner and assumed orientation
	 * to be PRINT_ORIENTATION_PORTRAIT.
	 *
	 * When orientation is set to PRINT_ORIENTATION_LANDSCAPE the
	 * geometry values still represent their values as if the
	 * orientation was PRINT_ORIENTATION_PORTRAIT.
	 */
	gint output_x, output_y;
	gint output_width, output_height;


	/* Buttons. */
	GtkWidget *print_btn, *cancel_btn;


	/* Keyboard modifiers mask. */
#define PRINT_MODIFIER_ALT	(1 << 0)
#define PRINT_MODIFIER_CTRL	(1 << 1)
#define PRINT_MODIFIER_SHIFT	(1 << 2)
	guint modifiers;

	/* Preview drag mode. */
#define PRINT_DRAG_NONE		0
#define PRINT_DRAG_TRANSLATE	1
#define PRINT_DRAG_SIZE		2
	gint drag_mode;
	gint drag_last_x, drag_last_y;

	/* Rectangular drag outline in window coordinates, for drawing
	 * a rectangular rubber band on the preview widget while dragging.
	 */
	gbool only_draw_outline;	/* Set by PrintWinPreviewEventCB() and
					 * cleared by PrintWinPreviewExposeCB()
					 */
	gint drag_outline_x[2], drag_outline_y[2];
	gbool have_prev_outline;
	gint prev_outline_x[2], prev_outline_y[2];

} print_win_struct;



/* Actually private, do not use these publically!! */
extern print_win_struct *PrintWinNew(imgview_struct *iv);
extern void PrintWinMap(print_win_struct *pw);
extern void PrintWinUnmap(print_win_struct *pw);
extern void PrintWinDelete(print_win_struct *pw);

/* Public. */
extern imgview_image_struct *PrintWinCreateImageFromValues(
        imgview_image_struct *src_img, print_values_struct *v
);
extern gbool PrintWinGetResponse(
        imgview_struct *iv,
        GtkWidget *ref_toplevel,        /* Reference toplevel. */
	print_values_struct *v
);


#endif	/* PRINTOPTS_H */
